@@ -1,3 +1,4 @@
+# PYTHON_ARGCOMPLETE_OK
 """
 IPython: tools for interactive and parallel computing in Python.
 
@@ -18,7 +19,6 @@ https://ipython.org
 # Imports
 #-----------------------------------------------------------------------------
 
-import os
 import sys
 
 #-----------------------------------------------------------------------------
@@ -38,7 +38,7 @@ Python 3.7 was still supported with the 7.x branch.
 
 See IPython `README.rst` file for more information:
 
-    https://github.com/ipython/ipython/blob/master/README.rst
+    https://github.com/ipython/ipython/blob/main/README.rst
 
 """
     )
