@@ -173,7 +173,7 @@ def _format_traceback_lines(lines, Colors, has_colors: bool, lvals):
 
 def _format_filename(file, ColorFilename, ColorNormal, *, lineno=None):
     """
-    Format filename lines with `In [n]` if it's the nth code cell or `File *.py` if it's a module.
+    Format filename lines with custom formatting from caching compiler or `File *.py` by default
 
     Parameters
     ----------
@@ -184,20 +184,29 @@ def _format_filename(file, ColorFilename, ColorNormal, *, lineno=None):
         ColorScheme's normal coloring to be used.
     """
     ipinst = get_ipython()
-
-    if ipinst is not None and file in ipinst.compile._filename_map:
-        file = "[%s]" % ipinst.compile._filename_map[file]
-        tpl_link = f"Input {ColorFilename}In {{file}}{ColorNormal}"
+    if (
+        ipinst is not None
+        and (data := ipinst.compile.format_code_name(file)) is not None
+    ):
+        label, name = data
+        if lineno is None:
+            tpl_link = f"{{label}} {ColorFilename}{{name}}{ColorNormal}"
+        else:
+            tpl_link = (
+                f"{{label}} {ColorFilename}{{name}}, line {{lineno}}{ColorNormal}"
+            )
     else:
-        file = util_path.compress_user(
+        label = "File"
+        name = util_path.compress_user(
             py3compat.cast_unicode(file, util_path.fs_encoding)
         )
         if lineno is None:
-            tpl_link = f"File {ColorFilename}{{file}}{ColorNormal}"
+            tpl_link = f"{{label}} {ColorFilename}{{name}}{ColorNormal}"
         else:
-            tpl_link = f"File {ColorFilename}{{file}}:{{lineno}}{ColorNormal}"
+            # can we make this the more friendly ", line {{lineno}}", or do we need to preserve the formatting with the colon?
+            tpl_link = f"{{label}} {ColorFilename}{{name}}:{{lineno}}{ColorNormal}"
 
-    return tpl_link.format(file=file, lineno=lineno)
+    return tpl_link.format(label=label, name=name, lineno=lineno)
 
 #---------------------------------------------------------------------------
 # Module classes
@@ -239,7 +248,7 @@ class TBTools(colorable.Colorable):
         self.debugger_cls = debugger_cls or debugger.Pdb
 
         if call_pdb:
-            self.pdb = debugger_cls()
+            self.pdb = self.debugger_cls()
         else:
             self.pdb = None
 
@@ -463,34 +472,25 @@ class ListTB(TBTools):
 
         Colors = self.Colors
         list = []
-        for filename, lineno, name, line in extracted_list[:-1]:
-            item = "  %s in %s%s%s\n" % (
-                _format_filename(
-                    filename, Colors.filename, Colors.Normal, lineno=lineno
-                ),
-                Colors.name,
-                name,
-                Colors.Normal,
+        for ind, (filename, lineno, name, line) in enumerate(extracted_list):
+            normalCol, nameCol, fileCol, lineCol = (
+                # Emphasize the last entry
+                (Colors.normalEm, Colors.nameEm, Colors.filenameEm, Colors.line)
+                if ind == len(extracted_list) - 1
+                else (Colors.Normal, Colors.name, Colors.filename, "")
             )
+
+            fns = _format_filename(filename, fileCol, normalCol, lineno=lineno)
+            item = f"{normalCol}  {fns}"
+
+            if name != "<module>":
+                item += f" in {nameCol}{name}{normalCol}\n"
+            else:
+                item += "\n"
             if line:
-                item += '    %s\n' % line.strip()
+                item += f"{lineCol}    {line.strip()}{normalCol}\n"
             list.append(item)
-        # Emphasize the last entry
-        filename, lineno, name, line = extracted_list[-1]
-        item = "%s  %s in %s%s%s%s\n" % (
-            Colors.normalEm,
-            _format_filename(
-                filename, Colors.filenameEm, Colors.normalEm, lineno=lineno
-            ),
-            Colors.nameEm,
-            name,
-            Colors.normalEm,
-            Colors.Normal,
-        )
-        if line:
-            item += '%s    %s%s\n' % (Colors.line, line.strip(),
-                                      Colors.Normal)
-        list.append(item)
+
         return list
 
     def _format_exception_only(self, etype, value):
@@ -616,6 +616,8 @@ class VerboseTB(TBTools):
     traceback, to be used with alternate interpreters (because their own code
     would appear in the traceback)."""
 
+    _tb_highlight = "bg:ansiyellow"
+
     def __init__(
         self,
         color_scheme: str = "Linux",
@@ -648,10 +650,8 @@ class VerboseTB(TBTools):
         self.long_header = long_header
         self.include_vars = include_vars
         # By default we use linecache.checkcache, but the user can provide a
-        # different check_cache implementation.  This is used by the IPython
-        # kernel to provide tracebacks for interactive code that is cached,
-        # by a compiler instance that flushes the linecache but preserves its
-        # own code cache.
+        # different check_cache implementation.  This was formerly used by the
+        # IPython kernel for interactive code, but is no longer necessary.
         if check_cache is None:
             check_cache = linecache.checkcache
         self.check_cache = check_cache
@@ -687,7 +687,7 @@ class VerboseTB(TBTools):
 
         func = frame_info.executing.code_qualname()
         if func == "<module>":
-            call = tpl_call.format(file=func, scope="")
+            call = ""
         else:
             # Decide whether to include variable details or not
             var_repr = eqrepr if self.include_vars else nullrepr
@@ -731,7 +731,7 @@ class VerboseTB(TBTools):
         if lvals_list:
             lvals = '%s%s' % (indent, em_normal.join(lvals_list))
 
-        result = "%s, %s\n" % (link, call)
+        result = f'{link}{", " if call else ""}{call}\n'
 
         result += ''.join(_format_traceback_lines(frame_info.lines, Colors, self.has_colors, lvals))
         return result
@@ -842,7 +842,7 @@ class VerboseTB(TBTools):
         before = context - after
         if self.has_colors:
             style = get_style_by_name("default")
-            style = stack_data.style_with_executing_node(style, "bg:ansiyellow")
+            style = stack_data.style_with_executing_node(style, self._tb_highlight)
             formatter = Terminal256Formatter(style=style)
         else:
             formatter = None
